# -*- coding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2010 Nexedi SA and Contributors. All Rights Reserved.
#
# WARNING: This program as such is intended to be used by professional
# programmers who take the whole responsibility of assessing all potential
# consequences resulting from its eventual inadequacies and bugs
# End users who are looking for a ready-to-use solution with commercial
# guarantees and support are strongly adviced to contract a Free Software
# Service Company
#
# This program is Free Software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#
##############################################################################

from AccessControl import ClassSecurityInfo
from Products.ERP5Type.XMLObject import XMLObject
from Products.ERP5Type import Permissions, PropertySheet

class ComposedDivergenceTester(XMLObject):
  """
  The purpose of this divergence tester is to group several testes to
  make a tester atomic per a target solver.
  """
  meta_type = 'ERP5 Composed Divergence Tester'
  portal_type = 'Composed Divergence Tester'
  add_permission = Permissions.AddPortalContent

  # Declarative security
  security = ClassSecurityInfo()
  security.declareObjectProtected(Permissions.AccessContentsInformation)

  # Declarative properties
  property_sheets = (   PropertySheet.Base
                      , PropertySheet.XMLObject
                      , PropertySheet.CategoryCore
                      , PropertySheet.DublinCore
                      , PropertySheet.DivergenceTester
                      , PropertySheet.SolverSelection
                     )

  def test(self, simulation_movement):
    """
    If one of inside testers says divergent, then this method returns
    False, otherwise it returns True.
    """
    for tester in self.objectValues():
      if not tester.test(simulation_movement):
        return False
    return True

  def explain(self, simulation_movement):
    """
    Returns a list of messages which come from inside testers.
    """
    message_list = []
    for tester in self.objectValues():
      message = tester.explain(simulation_movement)
      message_list.extend(message)
    return message_list

  def getTestedPropertyList(self, default=None):
    """
    Returns all tested properties in inside testers.
    """
    return sum([x.getTestedPropertyList() for x in self.objectValues()], [])
